// ADS Smart I/O
//
//   If you're not running this on a GC+, change the platform.
//
//   Uncomment the desired tests in main().
//
//   Compile it like this:
//     arm-linux-g++ -O2 -I. -L. sio.cc -lsio -o siotest
//
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "libsio.h"

// Platform - default is GC+
#define BITSY 0
#define GM    0 // must be 0 for GC+
#define OPT_3 0 // not available on all platforms and firmware versions

// Local prototypes
void print_device_info(void);
void test_back_light(void);
void test_keypad(void);
void test_adc(void);
void test_dio(int dir, int val); // Note hex args in main
void test_ucb_dio(void);
void test_ucb_adc(void);

#define GETBIT(buf,bit) (((buf) & (1 << (bit))) ? 1 : 0)

int main(int argc, char *argv[])
{
  print_device_info();

  test_back_light();
  //test_keypad();
  //test_adc();
  //test_dio((argc > 1) ? strtoul(argv[1], NULL, 0x10) : 0, (argc > 2) ? strtoul(argv[2], NULL, 0x10) : 0 );
  //test_ucb_adc();
  //test_ucb_dio();

  return 0;
}

void print_device_info(void)
{
  printf("Device Version  : %c\n", SIOReadDeviceVersion());
  printf("Device Part Type: %x\n", SIOReadDevicePartType());
  printf("Firmware Version: %x\n", SIOReadFirmwareVersion());
  sleep(1);
}

void test_back_light(void)
{
  int i;

 SIOBackLightOff();
  sleep(1);
  SIOBackLightOn();
  for (i = 0xff; i >= 0; i--) {
    SIOSetBackLightPwm(i);
  }
}

// Digital IO, Keypad, & A/D ---------------------------------------

#if BITSY
#  define SIO_DIG_A 0x0f
#  define SIO_DIG_B 0x00
#  define SIO_DIG_C 0x0f
#  define SIO_DIG_D 0x00
#  define MAX_A_PORTS 4
#  define MAX_B_PORTS 0
#  define MAX_C_PORTS 4 // depends on hardware options
#  define MAX_D_PORTS 0 // depends on hardware options
#else
#  define SIO_DIG_A 0xff
#  define SIO_DIG_B 0x0f
#  define SIO_DIG_C 0xff
#  define MAX_A_PORTS 8
#  define MAX_B_PORTS 4
#  define MAX_C_PORTS 8
#  if GM
#    define SIO_DIG_D 0x7f
#    define MAX_D_PORTS 7
#  else
#    define SIO_DIG_D 0x03
#    define MAX_D_PORTS 2
#  endif
#endif

void test_dio(int dir, int val)
{
  int buf, i, j, k = dir ? 2 : 1;

#if OPT_3
  SIOSelectOption(3); // all digital I/O
#else
  //  Option 3 NOT supported by AVR
  SIOSelectOption(2); // A/D on port A, digital I/O on others
#endif

// Set direction of digital I/Os
// Turn on pull up resistors by writing 1s to inputs

#if OPT_3
  SIOInitDDRA(dir & SIO_DIG_A);
  SIOWritePortA(val & SIO_DIG_A);
#endif

#if MAX_B_PORTS
  SIOInitDDRB(dir & SIO_DIG_B);
  SIOWritePortB(val & SIO_DIG_B);
#endif

  SIOInitDDRC(dir & SIO_DIG_C);
  SIOWritePortC(val & SIO_DIG_C);

#if MAX_D_PORTS
  SIOInitDDRD(dir & SIO_DIG_D);
  SIOWritePortD(val & SIO_DIG_D);
#endif

  printf("\n");
  for (j=0; j < 60; j += k) {
    if (j % 22 == 0) {
#if OPT_3
#  if BITSY
      printf("A    C\n");
      printf("3210 3210\n");
#  elif GM
      printf("A        B    C        D\n");
      printf("76543210 3210 76543210 6543210\n");
#  else
      printf("A        B    C        D\n");
      printf("76543210 3210 76543210 10\n");
#  endif
#else
#  if BITSY
      printf("C\n");
      printf("3210\n");
#  elif GM
      printf("B    C        D\n");
      printf("3210 76543210 6543210\n");
#  else
      printf("B    C        D\n");
      printf("3210 76543210 10\n");
#  endif
#endif
    }

#if OPT_3
    buf = SIOReadPortA();
    SIOWritePortA((buf^dir) & SIO_DIG_A); // toggle outputs / sample & hold inputs
    for (i=MAX_A_PORTS-1; i>=0; i--)
      printf("%d", GETBIT(buf,i));

    printf(" ");
#endif

#if MAX_B_PORTS
    buf = SIOReadPortB();
    SIOWritePortB((buf^dir) & SIO_DIG_B); // toggle outputs / sample & hold inputs
    for (i=MAX_B_PORTS-1; i>=0; i--)
      printf("%d", GETBIT(buf,i));

    printf(" ");
#endif

    buf = SIOReadPortC();
    SIOWritePortC((buf^dir) & SIO_DIG_C); // toggle outputs / sample & hold inputs
    for (i=MAX_C_PORTS-1; i>=0; i--)
      printf("%d", GETBIT(buf,i));

#if MAX_D_PORTS
    printf(" ");

    buf = SIOReadPortD();
    SIOWritePortD((buf^dir) & SIO_DIG_D); // toggle outputs / sample & hold inputs
    for (i=MAX_D_PORTS-1; i>=0; i--)
      printf("%d", GETBIT(buf,i));
#endif

    printf("\n");
    sleep(k);
  }
}

#if BITSY
#define TEST_ROWS 2 // MAX 5
#define TEST_COLS 3 // MAX 4
#else
#define TEST_ROWS 6 // MAX 8
#define TEST_COLS 3 // MAX 8
#endif

void test_keypad(void)
{
  int buf;

  SIOInitDDRC(0);     // all inputs
  SIOWritePortC(0);   // all pull ups off
  SIOSelectOption(1); // keypad - unused rows/cols are digital I/O

  printf("Configure keypad (%d rows, %d cols)\n", TEST_ROWS, TEST_COLS);
  SIOSetKeypadSize(TEST_COLS,TEST_ROWS);

  printf("Configure keypad to wait for keypresses\n\n");
  SetKeypadTimeout(-1);

  int ok = 3;
  while (ok) {
    buf = SIOReadKeypad() & 0x3f;
    printf("Keypad %d = col %d, row %d\n", buf, buf/TEST_ROWS, buf%TEST_ROWS);
    sleep(1);
    if (0 == buf) {
      --ok;
      printf("Got 0 - %d more to quit.\n", ok);
    }
    else if (TEST_ROWS*TEST_COLS <= buf) {
      --ok;
      printf("Invalid key!  (%d more to quit.)\n", ok);
      sleep(1);
    }
  }
}

void test_adc(void)
{
  int buf, ch, i;

  SIOSelectOption(2);
  printf("Read A/D converters\n\n");
  while (1) {
    sleep(1);
    for (i=0; i<MAX_A_PORTS; i++) {
      ch = i;
      buf = SIOAnalogConversion(ch);
      printf("ch%d = %x\n", i, buf);
    }
    printf("\n");
  }
}

// UCB1200 ---------------------------------------------

void test_ucb_dio(void)
{
  unsigned short buf;
  int i, j;

  SIOInitUCB(0x3ff);
  printf("Write UCB1200 digital I/O\n\n");
  printf("9876543210\n");
  sleep(2);
  for (j=0; j < 10; j++) {
    buf = 1<<j;
    SIOWriteUCB(buf);
    buf = SIOReadUCBDigital();
    if ((unsigned)~0 == buf) {
      printf("----------\n");
      continue;
    }
    for (i=9; i>=0; i--)
      printf("%d", GETBIT(buf,i));
    printf("\n");
    sleep(5);
  }

  SIOWriteUCB(0);
  SIOInitUCB(0);
  //  SIOWriteUCB(0);
  printf("\nRead UCB1200 digital I/O\n\n");
  for (j=0; j < 120; j++) {
    if (j % 20 == 0) {
      printf("\n");
      sleep(2);
      SIOInitUCB(0x3ff);
      SIOWriteUCB(0x3ff);
      printf("9876543210\n");
    }
    SIOInitUCB(0);
    sleep(1);
    buf = SIOReadUCBDigital();
    if ((unsigned)~0 == buf) {
      printf("----------\n");
      continue;
    }
    for (i=9; i>=0; i--)
      printf("%d", GETBIT(buf,i));
    printf("\n");
  }
}

void test_ucb_adc(void)
{
  unsigned short buf, i;

  printf("Read UCB 1200 A/D converters\n\n");
  while (1) {
    sleep(1);
    for (i=0; i<4; i++) {
      buf = SIOReadUCBAnalog(i);
      if ((unsigned)~0 != buf)
        printf("ch%d = %d\n", i, buf);
    }
    printf("\n");
  }
}

// ADS Smart I/O
//  - listed by port
//
//            GC+ & GM*       Bitsy
//           -----------   -----------
// Dig A/D   Key  Jumper   Key  Jumper
// A0  ch0   Col0 J7  17   Col0 J10 27
// A1  ch1   Col1 J7  19   Col1 J10 29
// A2  ch2   Col2 J7  21   Col2 J10 31
// A3  ch3   Col3 J7  23   Col3 J10 33
// A4  ch4   Col4 J7  25
// A5  ch5   Col5 J7  27
// A6  ch6   Col6 J7  29
// A7  ch7   Col7 J7  31
// B0             J2  12
// B1             J2  14
// B2             J2  16
// B3             J2  18
// C0        Row7 J7  15   Row0 J10 17
// C1        Row6 J7  13   Row1 J10 19
// C2        Row5 J7  11   Row2 J10 21
// C3        Row4 J7   9   Row3 J10 23
// C4        Row3 J7   7   Row4 J10 25
// C5        Row2 J7   5
// C6        Row1 J7   3        J3  35
// C7        Row0 J7   1        J3  37
// D0             J2  20        J10 15
// D1             J2  22        J10 13
//
// ADS Smart I/O
//  - listed by jumper
//
//       GC+ & GM                Bitsy
// --------------------   --------------------
// Jumper  Dig A/D Key    Jumper  Dig A/D Key
// J2  12  B0             J3  35  C6
//     14  B1                 37  C7
//     16  B2             J10 13  D1
//     18  B3                 15  D0
//     20  D0                 17  C0      Row0
//     22  D1                 19  C1      Row1
// J7   1  C7      Row0       21  C2      Row2
//      3  C6      Row1       23  C3      Row3
//      5  C5      Row2       25  C4      Row4
//      7  C4      Row3       27  A0  ch0 Col0
//      9  C3      Row4       29  A1  ch1 Col1
//     11  C2      Row5       31  A2  ch2 Col2
//     13  C1      Row6       33  A3  ch3 Col3
//     15  C0      Row7
//     17  A0  ch0 Col0
//     19  A1  ch1 Col1
//     21  A2  ch2 Col2
//     23  A3  ch3 Col3
//     25  A4  ch4 Col4
//     27  A5  ch5 Col5
//     29  A6  ch6 Col6
//     31  A7  ch7 Col7
//
// * GM also has a D2-D6 on J20
